#!/bin/sh

# Default display name
DNAME="Aria2"

PACKAGE_BASE="/var/packages/${DNAME}/target"
SVC_CWD="${PACKAGE_BASE}"
ARIA2_BIN="${PACKAGE_BASE}/bin/aria2c"
ARIA2_CONF="${PACKAGE_BASE}/conf/aria2.conf"
LOG_FILE="${PACKAGE_BASE}/logs/${DNAME}.log"
PID_FILE="/var/packages/${DNAME}/var/${DNAME}.pid"
SERVICE_COMMAND[0]="${ARIA2_BIN} --conf-path=${ARIA2_CONF}"
UP_TRACKER_URL=`cat "${ARIA2_CONF}"|grep -w "# up-tracker-url"|sed s/[[:space:]]//g | sed "s/'//g" | sed 's/"//g'|awk -F '=' '{print $2}'|grep -v '^$'`

SVC_BACKGROUND=y
SVC_WRITE_PID=y


# Invoke shell function if available
call_func ()
{
    FUNC=$1
    if type "${FUNC}" 2>/dev/null | grep -q 'function' 2>/dev/null; then
        echo "Begin ${FUNC}" >> ${LOG_FILE}
        eval ${FUNC}         >> ${LOG_FILE}
        echo "End ${FUNC}"   >> ${LOG_FILE}
    fi
}


start_daemon ()
{
    date > ${LOG_FILE}

    if [ -z "`grep "up-tracker-url=" ${ARIA2_CONF}`" ]; then
        sed -i '$a # up-tracker-url=""' "${ARIA2_CONF}"
    fi
    if [ "${UP_TRACKER_URL}" != "" ]; then
        # 初始化跟踪器列表变量
        ARIA2_TRACKER_LIST=""

        # 循环处理每个 URL
        for TRACKER_URL in ${UP_TRACKER_URL}; do
            TRACKER_LIST=$(curl -s -k -m 3 "${TRACKER_URL}")
            [ "$(echo ${TRACKER_LIST}|grep -wc ',')" = 0 ] && TRACKER_LIST=$(echo "${TRACKER_LIST}" | awk NF | sed ':a;N;s/\n/,/g;ta')

            if [ -n "${TRACKER_LIST}" ]; then
                # 如果 ARIA2_TRACKER_LIST 已经有内容，则添加逗号
                [ -n "${ARIA2_TRACKER_LIST}" ] && ARIA2_TRACKER_LIST="${ARIA2_TRACKER_LIST},"
                ARIA2_TRACKER_LIST="${ARIA2_TRACKER_LIST}${TRACKER_LIST}"
            fi
        done

        # 处理 bt-tracker 配置
        if [ -z "$(grep "bt-tracker=" "${ARIA2_CONF}")" ]; then
            echo "添加 bt-tracker 设置..."  >> ${LOG_FILE}
            sed -i '$a bt-tracker='"${ARIA2_TRACKER_LIST//,/\\,}" "${ARIA2_CONF}"
        else
            echo "更新 bt-tracker 设置..." >> ${LOG_FILE}
            sed -i "s@bt-tracker=.*@bt-tracker=${ARIA2_TRACKER_LIST//,/\\,}@g" "${ARIA2_CONF}"
        fi
    fi

    declare -i i=0
    for service in "${SERVICE_COMMAND[@]}"
    do
        let i=i+1
        if [ -z "${SVC_QUIET}" ]; then
            echo "Starting ${DNAME} command ${service}" >> ${LOG_FILE}
        fi
        if [ -n "${service}" ]; then
            if [ -n "${SVC_NO_REDIRECT}" ]; then
                OUT="/dev/null"
            else
                OUT="${LOG_FILE}"
            fi
            if [ -n "${SVC_CWD}" ]; then
                cd "${SVC_CWD}"
            fi
            if [ -z "${SVC_BACKGROUND}" ]; then
                ${service} >> ${OUT} 2>&1
            else
                ${service} >> ${OUT} 2>&1 &
            fi

            if [ -n "${SVC_WRITE_PID}" -a -n "${SVC_BACKGROUND}" -a -n "${PID_FILE}" ]; then
                [ $i -eq 1 ] && echo -ne "$!" > ${PID_FILE} || echo -ne " $!" >> ${PID_FILE}
            else
                wait_for_status 0 ${SVC_WAIT_TIMEOUT:=20}
            fi
        fi
    done
}

stop_daemon ()
{
    if [ -n "${PID_FILE}" -a -r "${PID_FILE}" ]; then
        for pid in $(cat "${PID_FILE}")
        do
            if [ -z "${SVC_QUIET}" ]; then
                date >> ${LOG_FILE}
                echo "Stopping ${DNAME} service : $(ps -p${pid} -o comm=) (${pid})" >> ${LOG_FILE}
            fi
            if kill -0 ${pid} &> /dev/null; then
                kill -TERM ${pid} >> ${LOG_FILE} 2>&1
                wait_for_status 1 ${SVC_WAIT_TIMEOUT:=20} ${pid} || kill -KILL ${pid} >> ${LOG_FILE} 2>&1
            fi
        done
        if [ -f "${PID_FILE}" ]; then
            rm -f "${PID_FILE}" > /dev/null
        fi
    fi
}

#------------------------------------------------------
# daemon_status()
#     $1: PID to check, if empty use ${PID_FILE}
# status: Keeps track of kill -0 exit status
#
# Return 0 when all pid are OK, else return 1
#------------------------------------------------------
daemon_status ()
{
    status=0
    [ -z "${1}" ] && pid_list=$(cat ${PID_FILE} 2>/dev/null) || pid_list=${1}
    if [ -n "${pid_list}" ]; then
        for pid in ${pid_list}
        do
           kill -0 ${pid} &> /dev/null
           let status=$status+$?
        done
        if [ $status -ne 0 ]; then
           stop_daemon
           return 1
        else
           return 0
        fi
    else
        return 1
    fi
}

#------------------------------------------------------
# wait_for_status()
#      $1: expected return from daemon_status() call
#      $2: timeout (e.g. number of loop to be done)
#      $3: PID to check being passed to daemon_status()
# counter: Number of 1sec iteration to wait until
#          the return value from daemon_status()
#          match expected value
#
# Wait for a duration of $counter seconds for the
# return value from daemon_status().  If it match
# return 0 else return 1 if wait time is over.
#------------------------------------------------------
wait_for_status ()
{
    # default value: 20 seconds
    counter=${2}
    counter=${counter:=20}
    while [ ${counter} -gt 0 ]; do
        daemon_status ${3}
        [ $? -eq $1 ] && return
        let counter=counter-1
        sleep 1
    done
    return 1
}

stop_daemon && start_daemon